﻿define(["sitecore", "/-/speak/v1/odg/applib.js"], function (_sc, applib) {
  return _sc.Definitions.App.extend({
    initialized: function () {
      $.ajaxSetup({ cache: false });
      var app = this;

      this.on("item:save", this.saveTask, this);

      this.set("lastSelectedDesignId", null);
      this.on("change:lastSelectedDesignId", function () {
        this.InsertDesignButton.set("isEnabled", this.get("lastSelectedDesignId") != null);
      }, this);

      _sc.on("sc-frame-message", function (value) {
        var modal = this.SelectDesignDialogWindow.viewModel.$el.data('modal');
        if (modal && modal.isShown) {
          this.set("lastSelectedDesignId", value && value.length > 0 ? value : null);
        }
      }, this);

      this.SelectDesignDialogWindow.viewModel.$el.addClass("frameDialog");

      var collectionId = applib.getUrlParameter("cid");
      if (collectionId == "") {
        this.back();
      }

      this.DesignFrame.set("sourceUrl", applib.settings.navigation.url + "components/dialogs/selectdesigndialog?id=" + collectionId);
      this.SelectedDesignBorder.viewModel.$el.addClass("selected-itemIcon");
      applib.getItemFromDatabase(collectionId, function (collectionItem) {
        if (collectionItem == null) {
          app.back();
        }
        app.collectionItem = collectionItem;
        app.updateCollectionInfo();
      });

      var designId = applib.getUrlParameter("did");
      if (designId != "") {
        applib.getItemFromDatabase(designId, function (data) {
          app.selectedDesign = data;
          app.updateDesignInfo();
        }, this);
      }
      else {
        this.updateDesignInfo();
      }

      applib.getItemAccess(applib.settings.rootFolderIds.designTasks, function (data) {
        this.SaveButton.set("isEnabled", data.canCreate);
        this.SelectDesignButton.set("isEnabled", data.canCreate);
        this.canCreateTask = data.canCreate;
        if (!data.canCreate) {
          var errorMessage = this.StringDictionary.translate("You do not have permission to create an item here.");
          this.MessageBar.addMessage("warning", errorMessage);
        }
      }, this);
    },

    updateCollectionInfo: function () {
      this.CollectionHyperlinkButton.set("text", this.collectionItem.$displayName);
      this.CollectionHyperlinkButton.viewModel.$el.attr("title", this.collectionItem.$displayName);
      var collectionColumn = $("[data-sc-id=InfoValuesColumn0]");
      if (collectionColumn) {
        collectionColumn.addClass("iteminfo-name");
      }
    },

    updateDesignInfo: function () {
      if (this.selectedDesign != null) {
        this.SelectedDesignImage.set("imageUrl", this.selectedDesign.$mediaurl);
        this.SelectedDesignName.set("text", this.selectedDesign.$displayName);
        this.SelectedDesignName.viewModel.$el.attr("title", this.selectedDesign.$displayName);
        $("[data-sc-id=SelectedDesignBorder]").css("border-color", "");
      }
      else {
        this.SelectedDesignImage.set("imageUrl", "");
        this.SelectedDesignName.set("text", this.NoDesignSelected.get("text"));
        this.SelectedDesignName.viewModel.$el.attr("title", this.NoDesignSelected.get("text"));
      }
    },

    saveTask: function () {
      if (this.isSaving) {
        return;
      }

      this.MessageBar.removeMessages();
      var errorMessage;

      if (!this.canCreateTask) {
        errorMessage = this.StringDictionary.translate("You do not have permission to create an item here.");
        this.MessageBar.addMessage("error", errorMessage);
        return;
      }

      var name = this.NameTextBox.get("text");

      var isValid = name && name.length > 0;
      $("[data-sc-id=NameTextBox]").css("border-color", isValid ? "" : "red");
      if (!isValid) {
        errorMessage = this.StringDictionary.get("An item name cannot be blank.");
        if (!errorMessage || errorMessage.length == 0) {
          errorMessage = this.StringDictionary.translate("'$Input' is not a valid name.");
          errorMessage = errorMessage.replace("$Input", "");
        }
        this.MessageBar.addMessage('error', errorMessage);
      }

      if (this.selectedDesign == null) {
        var designWarning = this.DesignNotSelected.get("text");
        if (designWarning) {
          this.MessageBar.addMessage("error", this.DesignNotSelected.get("text"));
        }
        $("[data-sc-id=SelectedDesignBorder]").css("border-color", "red");

        isValid = false;
      }

      if (!isValid) {
        return;
      }

      var self = this;
      this.isSaving = true;
      this.ProgressIndicator.set("isBusy", true);


      _sc.Definitions.Data.createItem({
        templateId: applib.settings.templateIds.designTask,
        name: name,
        parentId: applib.settings.rootFolderIds.designTasks,
        database: applib.settings.masterDatabaseName
      }, {
        Name: name,
        Description: this.InfoTextBox.get("text"),
        'Design Collection': this.collectionItem.itemId,
        'Design Template': this.selectedDesign.itemId
      }, function (item) {

        $.ajax({
          url: applib.settings.navigation.commandRoutePrefix + "collection/updateusage",
          type: "PUT",
          dataType: "json",
          data: { itemId: item['Design Collection'] }
        });

        $.ajax({
          url: applib.settings.navigation.commandRoutePrefix + "design/updateusage",
          type: "PUT",
          dataType: "json",
          data: { itemId: item["Design Template"] }
        });

        $.ajax({
          url: applib.settings.navigation.commandRoutePrefix + "design/synctasklocalproject",
          type: "PUT",
          dataType: "json",
          data: { itemId: item.itemId, clone: true, sync: false }
        }).done(function (data) {
          self.goToTask(item.itemId);
        }).fail(function (err) {
          self.MessageBar.addMessage("error", err.responseText);
        });

      }).fail(function (response) {
        self.isSaving = false;
        self.ProgressIndicator.set("isBusy", false);

        errorMessage = self.StringDictionary.get(response.responseText);
        if (!errorMessage || errorMessage.length == 0) {
          errorMessage = self.StringDictionary.translate("'$Input' is not a valid name.");
          errorMessage = errorMessage.replace("$Input", name);
        }
        self.MessageBar.addMessage('error', errorMessage);
      });
    },

    openSelectDesign: function () {
      this.set("lastSelectedDesignId", null);
      this.SelectDesignDialogWindow.show();
    },

    selectDesignSelected: function () {
      this.SelectDesignDialogWindow.hide();
      var app = this;

      if (this.get("lastSelectedDesignId") != null) {
        applib.getItemFromDatabase(this.get("lastSelectedDesignId"), function (designItem) {
          app.selectedDesign = designItem;
          var designWarning = app.DesignNotSelected.get("text");
          if (app.selectedDesign != null && designWarning) {
            app.MessageBar.removeMessage(function (warning) {
              return warning.text === designWarning;
            });
          }
          app.updateDesignInfo();
        });
      }
    },

    viewCollection: function () {
      window.location = applib.settings.navigation.url + "salesviewcollection?id=" + this.collectionItem.itemId + "&ref=adddocument";
    },

    back: function () {
      var ref = applib.getUrlParameter("ref");
      if (ref == "") {
        ref = "DocumentPublisher";
      }

      window.location = applib.settings.navigation.url + ref;
    },

    goToTask: function (id) {
      var ref = applib.getUrlParameter("ref");
      if (ref == "") {
        ref = "DocumentPublisher";
      }

      window.location = applib.settings.navigation.url + "editdocument?id=" + id + "&ref=" + ref + "&new=1";
    }
  });
});