﻿define(["sitecore"], function (_sc) {

  var appHelper = _.extend({

    database: this.base == null ? this.base = new _sc.Definitions.Data.Database(new _sc.Definitions.Data.DatabaseUri("master")) : this.base,

    getUrl: function () {
      var itemUri = this.get("itemUri"),
        database = new _sc.Definitions.Data.Database(itemUri.getDatabaseUri()),
        url = database.getUrl(itemUri.getItemId());

      url = _sc.Helpers.url.addQueryParameters(url, {
        language: appHelper.contentLanguage
      });

      return url;
    },

    getUrlParameter: function (parameterName) {
      return this.getParameter(parameterName, document.URL);
    },

    getParameter: function (parameterName, url) {
      var hash;
      var q = url.split('?')[1];
      if (q != undefined) {
        q = q.split('&');
        for (var i = 0; i < q.length; i++) {
          hash = q[i].split('=');
          if (hash[0] == parameterName) {
            return hash[1];
          }
        }
      }

      return "";
    },

    isUpdatedItem: function (itemId, updateUrl) {
      var updatedItemId = this.getParameter("sc_itemid", updateUrl);
      return decodeURIComponent(updatedItemId) == itemId;
    },

    getItemFromDatabase: function (itemId, completed, lang) {
      if (itemId == "") {
        return;
      }

      var options = {};
      options.payLoad = "full";
      options.formatting = "$send_localized_dates";
      if (lang) {
        options.language = this.contentLanguage || "";
      }

      this.database.getItem(itemId, completed, options);
    },

    getItemParent: function (itemId, completed, lang) {
      if (itemId == "") {
        return;
      }

      var options = {};
      options.payLoad = "full";
      options.scope = ["parent"];
      if (lang) {
        options.language = this.contentLanguage || "";
      }

      this.database.getItem(itemId, completed, options);
    },

    getChildren: function (itemId, completed, lang) {
      if (itemId == "") {
        return;
      }

      var options = {};
      options.payLoad = "full";
      options.scope = ["children"];
      if (lang) {
        options.language = this.contentLanguage || "";
      }

      this.database.getChildren(itemId, completed, options);
    },

    getItemFromList: function (listControl, completed, scope) {
      var selectedItem = listControl.get("selectedItem");
      if (!selectedItem)
        return;

      var fields = selectedItem.get("$fields");
      if (fields != null) {
        completed.call(scope, fields[0].item);
      } else {
        this.getItemFromDatabase(selectedItem.get("itemId"), function (item) {
          if (item != null) {
            completed.call(scope, item);
          }
        });
      }
    },

    getDateValue: function (item, fieldName) {
      if (!item.$fields) return null;

      var field = _.findWhere(item.$fields, { fieldName: fieldName });
      if (field && field.shortDateValue) {
        return field.shortDateValue;
      }
      
      var dateConverter = _sc.Converters.get("date");
      var fieldValue = item[fieldName];
      if (fieldValue && fieldValue.length > 0) {
        return dateConverter.toStringWithFormat(fieldValue, this.settings.formatting.dateFormat);
      }
      
      return "";
    },

    search: function (database, searchText, completed, options) {
      if (!completed) {
        throw "Parameter 'completed' is null";
      }

      var url = database.getUrl("search:" + searchText, options);

      if (options.itemId && options.itemId.length > 0) {
        url = _sc.Helpers.url.addQueryParameters(url, { "odg_itemId": options.itemId });

        if (options.listField && options.listField.length > 0) {
          url = _sc.Helpers.url.addQueryParameters(url, { "odg_listField": options.listField });
        }
      }

      database.get(url).pipe(database.convertToItems).done(function (data) {
        completed(data.items, data.total, data.data);
      }).fail(function (err) {
        completed([], 0, err);
      });
    },

    getItemAccess: function (itemId, completed, scope) {
      var ajaxOptions = {
        url: this.settings.navigation.commandRoutePrefix + "design/getitemaccess",
        type: "GET",
        dataType: "json",
        data: { itemId: itemId }
      };

      $.ajax(ajaxOptions).done(function (data) {
        completed.call(scope, data);
      });
    },

    addDialog: function (app, renderingId, dialogId, completed) {
      var temp = app;
      app.insertRendering(renderingId, { path: this.settings.renderings.renderingRootPath, $el: $("body") }, function (subApp) {
        temp[dialogId] = subApp;
        if (completed) {
          completed.call(app);
        }
      });
    },

    setItemsListValue: function (fieldValue, collection) {
      if (fieldValue) {
        try {
          var itemIds = fieldValue.split("|");
          for (var n = 0; n < itemIds.length; n++) {
            this.addToCollection(itemIds[n], collection);
          }
        } catch (e) {
        }
      }
    },

    addToCollection: function (itemId, collection) {
      this.getItemFromDatabase(itemId, function (dataItem) {
        collection.add(dataItem);
      });
    },

    getCollectionState: function (collectionId, completed) {
      $.ajax({
        url: this.settings.navigation.commandRoutePrefix + "collection/getcollectionstate",
        dataType: "json",
        data: { itemId: collectionId }
      }).done(function (data) {
        completed(data);
      });
    },

    getDate: function (date) {
      var cleanDate = new Date(date.getFullYear(), date.getMonth(), date.getDate());
      return cleanDate;
    },

    getEditorTypes: function (callback) {
      if (this.editorTypes == null) {
        var self = this;
        $.get(this.settings.navigation.commandRoutePrefix + "editortypes/alleditortypes").done(function (data) {
          self.editorTypes = data;
          callback(self.editorTypes);
        });
      }
      else {
        callback(this.editorTypes);
      }
    },

    getEditorType: function (key, callback) {
      var editorType;
      this.getEditorTypes(function (editorTypes) {
        editorType = _.find(editorTypes, function (type) { return type.key == key; });
        callback(editorType);
      });
    },

    getEditorTypesQuery: function (callback) {
      if (this.editorTypesQuery == null) {
        var self = this;
        this.getEditorTypes(function (editorTypes) {
          var keys = _.map(editorTypes, function (type) { return "@@templatekey='" + type.key.toLowerCase() + "'"; });
          self.editorTypesQuery = keys.join(" or ");
          callback(self.editorTypesQuery);
        });
      }
      else {
        callback(this.editorTypesQuery);
      }
    },

    settings: {
      masterDatabaseName: 'master',

      fieldIds: {
        createdDate: '{25BED78C-4957-4165-998A-CA1B52F67497}',
        createdBy: '{5DD74568-4D4B-44C1-B513-0AF5F4CDA34F}'
      },

      renderings: {
        renderingRootPath: '/sitecore/client/',
        createCollectionDialog: '{A5B0B780-6B25-4A88-B74F-F8E2D2EB8C36}',
        assignDataDialog: '{719FB94A-96A1-4E47-8C11-45EF8D4D051D}',
        assignImageDialog: '{68AD2FE3-A932-426D-8F02-8D4ADD62EBD7}',
        editDesignDialog: '{8136486D-CFB5-42BB-B6D7-BE64E6E6B3F2}',
        selectDesignDialog: '{77B44619-4C6F-4BE3-82FD-45557C52CD8D}',
        designEdit: '{6090A951-7365-4E51-B2D6-5B12570717A0}',
        textEdit: '{CD8E4F2F-A6A4-4DEC-9406-39DE5C4E9C1F}',
        imageEdit: '{8CF4552A-2822-4A32-9E06-84C70E6C6A78}',
        dataEdit: '{405DF344-671F-4EC3-AA07-6EFD0E39FD42}',
        snippetEdit: '{22B0B122-F51B-4F71-A21D-A956F55B148F}',
        dataTreeView: '{DBA6C9F8-3235-4F10-974E-9B9D7B2C3149}'
      },

      templateIds: {
        designTask: '{945CAF2F-F5E1-4412-B879-7496D0B6D25B}',
        designCollection: '{6992ADAE-1D72-4299-BC6E-C7E6820D20BD}'
      },

      rootFolderIds: {
        designCollections: '{25EAEE5F-9629-4238-814D-B9468196089A}',
        designTasks: '{C5C15359-B7FE-411C-A5ED-A66573B83C81}'
      },

      formatting: {
        datePickerFormat: "dd-mm-yy",
      },

      navigation: {
        url: "/sitecore/client/applications/odg/",
        commandRoutePrefix: "/api/sitecore/odg/"
      }
    }
  });

  return appHelper;
});

function createCollectionDialog() {
  _sc.trigger("createDialog");
}