﻿/// <reference path="../../../../../../assets/lib/dist/sitecore.js" />

define(["sitecore", "/-/speak/v1/odg/applib.js"], function (_sc, applib) {
  return _sc.Definitions.App.extend({
    initialized: function () {
      $.ajaxSetup({ cache: false });
      this.on("closePopup", this.closePopup, this);
      _sc.on("view:collection", this.viewCollection, this);

      applib.addDialog(this, applib.settings.renderings.createCollectionDialog, "createDialog");

      this.CollectionsListControl.on("change:selectedItemId", this.setInfoPanel, this);

      var linkGuid = applib.getUrlParameter("sc");
      if (!this.setSearchConfigId(linkGuid)) {
        this.setSearchConfigId(this.DefaultSearchConfigIdText.get("text"));
      }

      if (this.CreateCollectionButton) {
        applib.getItemAccess(applib.settings.rootFolderIds.designCollections, function (data) {
          this.CreateCollectionButton.set("isEnabled", data.canCreate);
        }, this);
      }

      if (this.CreateTaskButton) {
        applib.getItemAccess(applib.settings.rootFolderIds.designTasks, function (data) {
          this.CreateTaskButton.set("isEnabled", data.canCreate);
          this.canCreateTask = data.canCreate;
        }, this);
      }
    },

    setSearchConfigId: function (guid) {
      var isGuid = guid && _sc.Helpers.id.isId(guid);
      if (isGuid) {
        this.DataSource.set("searchConfig", guid);
      }

      return isGuid;
    },

    setInfoPanel: function () {
      var selected = this.CollectionsListControl.get("selectedItemId");
      if (selected && selected.length > 0) {
        applib.getItemFromList(this.CollectionsListControl, this.fillItemInfo, this);
      }
    },

    fillItemInfo: function (item) {
      this.ItemInfoContent.set("item", item);
    },

    editCollection: function () {
      var itemId = this.CollectionsListControl.get("selectedItemId");
      if (itemId && itemId.length > 0) {
        window.location = applib.settings.navigation.url + "editcollection?id=" + itemId + "&ref=collections";
      }
    },

    viewCollection: function (itemId) {
      if (itemId && itemId.length > 0) {
        var isSalesPage = location.pathname.substring(location.pathname.lastIndexOf("/") + 1).toLowerCase() == "salescollections";
        if (isSalesPage) {
          window.location = applib.settings.navigation.url + "salesviewcollection?id=" + itemId + "&ref=salescollections";
          return;
        }
        window.location = applib.settings.navigation.url + "viewcollection?id=" + itemId + "&ref=collections";
      }
    },

    deleteCollection: function () {
      this.MessageBar.removeMessages();

      var selectedItems = this.CollectionsListControl.get("checkedItems");
      applib.getItemFromList(this.CollectionsListControl, function (data) {
        if (data != null) selectedItems = [data];
      }, this);
      if (!selectedItems || selectedItems.length == 0) {
        return;
      }

      var deleteConfirmationText;
      if (selectedItems.length == 1) {
        deleteConfirmationText = _sc.Helpers.string.format(this.StringDictionary.translate("Are you sure you want to delete \"{0}\"?"), selectedItems[0].$displayName);
      }
      else {
        deleteConfirmationText = _sc.Helpers.string.format(this.StringDictionary.translate("Are you sure you want to delete these {0} items?{1}"), selectedItems.length, "");
      }

      if (confirm(deleteConfirmationText)) {
        var totalCount = selectedItems.length;
        var noPermissionMessage = this.StringDictionary.translate("You do not have permission to delete \"{0}\".");
        var deleteItemIds = [];
        _.each(selectedItems, function (selItem) {
          applib.getItemAccess(selItem.itemId, function (data) {
            if (data.canDelete) {
              deleteItemIds.push(selItem.itemId);
            } else {
              this.MessageBar.addMessage("warning", _sc.Helpers.string.format(noPermissionMessage, selItem.$displayName));
            }
            if (!--totalCount) {
              this.doDeleteCollections(deleteItemIds);
            }
          }, this);
        }, this);
      }
    },

    doDeleteCollections: function (deleteItemIds) {
      var self = this;
      var deleteCount = deleteItemIds.length;

      var refreshDataSource = function (data) {
        /*
        if (data && data.statusCode !== 200 && data.error && data.error.message.length > 0) {
          self.MessageBar.addMessage("error", data.error.message);
        }
        */
        if (!--deleteCount) {
          self.CollectionsListControl.set("selectedItemId", null);
          self.DataSource.refresh();
        }
      };

      _.each(deleteItemIds, function (itemId) {
        applib.getItemFromDatabase(itemId, function (item) {
          if (item != null) {
            item.toModel().remove(refreshDataSource, self).fail(refreshDataSource, self);
          } else {
            refreshDataSource.call(self, null);
          }
        });
      }, this);
    },

    createTask: function () {
      var itemId = this.CollectionsListControl.get("selectedItemId");
      if (itemId && itemId.length > 0 && this.canCreateTask) {
        var self = this;
        applib.getCollectionState(itemId, function (result) {
          if (result.isActive) {
            window.location = applib.settings.navigation.url + "adddocument?cid=" + itemId + "&ref=salescollections";
          }
          else {
            alert(self.CollectionNotRunningText.get("text"));
          }
        });
      }
    }
  });
});