﻿define(["sitecore"], function (_sc) {
  _sc.Factories.createBaseComponent({
    name: "ContentLanguageSwitcher",
    base: "BlockBase",
    selector: ".sc-ContentLanguageSwitcher",

    attributes: [
     { name: "contentLanguage", value: "$el.data:sc-contentlanguage" },
     { name: "isOpen", defaultValue: false }
    ],

    initialize: function () {
      this._super();
      this.model.on("change:contentLanguage", this.setSelectedLanguage, this);
      this.model.on("change:isOpen", this.openChange, this);
      var self = this;
      this.documentClick = function (evt) {
        if (self.model.get("isOpen")) {
          var parent = $(evt.target).closest('div[class^="sc-ContentLanguageSwitcher"]');
          if (parent.length == 0) {
            self.model.set("isOpen", false);
          }
        }
      };

      this.$el.find("ul.nav").removeClass("hide");
    },

    openChange: function () {
      var isOpen = this.model.get("isOpen");
      if (isOpen) {
        $(document).on("click", this.documentClick);
      }
      else {
        $(document).off("click", this.documentClick);
      }
    },

    toggleIsOpen: function () {
      this.model.set("isOpen", !this.model.get("isOpen"));
    },

    select: function (data, event) {
      var source = $(event.target);
      if (!source.data("isocode")) {
        source = source.parents("[data-isocode]");
      }

      if (source != null && source.data("isocode")) {
        var isoCode = source.data("isocode");

        if (this.model.get("contentLanguage") == isoCode) {
          this.model.set("isOpen", false);
        } else {
          this.model.set("contentLanguage", isoCode);
        }
      }
    },

    setSelectedLanguage: function () {
      var isoCode = this.model.get("contentLanguage");
      if (!isoCode) {
        return;
      }

      this.model.set("isOpen", false);
      this.$el.find(".selected").removeClass("selected");

      var languageLink = this.$el.find("a.language-item[data-isocode=\'" + isoCode + "\']");
      var li = languageLink.closest("li");

      var toggleButton = this.$el.find(".language-switcher-btn");
      toggleButton.attr('title', li.attr('title') || isoCode);
      toggleButton.find(".language-switcher-icon").attr("style", languageLink.find(".sc-icon").attr('style') || "");
      toggleButton.find(".sc-dropdownbutton-text").text(languageLink.data("shortname") || "");
      li.addClass("selected");
    },

    hasLanguage: function (isoCode) {
      if (!isoCode || !this.$el.find("a.language-item[data-isocode=\'" + isoCode + "\']")) {
        return false;
      }

      return true;
    }
  });
});