﻿define(["sitecore"], function (_sc) {
  var attributesEdit = _sc.Definitions.App.extend({
    initialized: function () {

      this.set("item", null);
      this.on("change:item", this.loadItem, this);
      this.WidthTextBox.on("change:text", this.changeWidth, this);
      this.HeightTextBox.on("change:text", this.changeHeight, this);
      this.XTextBox.on("change:text", this.changeX, this);
      this.YTextBox.on("change:text", this.changeY, this);
    },

    changeWidth: function () {
      if (!this.isValidNumber(this.WidthTextBox.get("text"))) {
        $("[data-sc-id=WidthTextBox]").css("border-color", "red");
      }
      else {
        $("[data-sc-id=WidthTextBox]").css("border-color", "");
      }

      var item = this.get("item");
      if (item["Width"] != this.WidthTextBox.get("text")) {
        item["Width"] = this.WidthTextBox.get("text");
      }
    },

    changeHeight: function () {
      if (!this.isValidNumber(this.HeightTextBox.get("text"))) {
        $("[data-sc-id=HeightTextBox]").css("border-color", "red");
      }
      else {
        $("[data-sc-id=HeightTextBox]").css("border-color", "");
      }

      var item = this.get("item");
      if (item["Height"] != this.HeightTextBox.get("text")) {
        item["Height"] = this.HeightTextBox.get("text");
      }
    },

    changeX: function () {
      if (!this.isValidNumber(this.XTextBox.get("text"))) {
        $("[data-sc-id=XTextBox]").css("border-color", "red");
      }
      else {
        $("[data-sc-id=XTextBox]").css("border-color", "");
      }

      var item = this.get("item");
      if (item["X"] != this.XTextBox.get("text")) {
        item["X"] = this.XTextBox.get("text");
      }
    },

    changeY: function () {
      if (!this.isValidNumber(this.YTextBox.get("text"))) {
        $("[data-sc-id=YTextBox]").css("border-color", "red");
      }
      else {
        $("[data-sc-id=YTextBox]").css("border-color", "");
      }

      var item = this.get("item");
      if (item["Y"] != this.YTextBox.get("text")) {
        item["Y"] = this.YTextBox.get("text");
      }
    },

    loadItem: function () {
      var item = this.get("item");
      if (item != null) {
        this.originalWidth = item["Width"];
        this.originalHeight = item["Height"];
        this.originalX = item["X"];
        this.originalY = item["Y"];

        this.WidthTextBox.set("text", item["Width"]);
        this.HeightTextBox.set("text", item["Height"]);
        this.XTextBox.set("text", item["X"]);
        this.YTextBox.set("text", item["Y"]);
      }
    },

    acceptChanges: function () {
      var item = this.get("item");

      this.originalWidth = item["Width"];
      this.originalHeight = item["Height"];
      this.originalX = item["X"];
      this.originalY = item["Y"];
    },

    hasChanges: function () {
      var item = this.get("item");

      if (this.originalWidth != item["Width"]
       || this.originalHeight != item["Height"]
       || this.originalX != item["X"]
       || this.originalY != item["Y"]) {
        return true;
      }

      return false;
    },

    isValid: function () {

      var item = this.get("item");

      if (!this.isValidNumber(item["Width"])
       || !this.isValidNumber(item["Height"])
       || !this.isValidNumber(item["X"])
       || !this.isValidNumber(item["Y"])) {
        return false;
      }

      return true;
    },

    isValidNumber: function (value) {
      var regEx = /^-{0,1}\d*\.{0,1}\d+$/;
      return (regEx.test(value));
    }
  });

  return attributesEdit;
});