﻿define(["sitecore", "/-/speak/v1/odg/applib.js"], function (_sc, applib) {

  var model = _sc.Definitions.Models.ControlModel.extend({
    initialize: function (options) {
      this._super();

      this.rectangles = {};

      this.pageWidth = "";
      this.set("imageUrl", "");
      this.set("pageId", "");
      this.set("item", null);
      this.set("items", null);
      this.image = null;
      this.objectsLoading = false;

      this.on("change:item", this.highlightItem, this);
      this.on("change:imageUrl", this.loadImage, this);
    },

    loadImage: function () {
      var canvas = this.viewModel.$el[0];
      var context = canvas.getContext('2d');
      context.clearRect(0, 0, context.canvas.width, context.canvas.height);
      context.beginPath();

      if (this.get("imageUrl") != "") {
        var that = this;
        var image = new Image();
        image.onload = function () {
          canvas.width = this.width;
          canvas.height = this.height;
          that.image = image;
          that.imageLoaded = true;
          context.drawImage(image, 0, 0, this.width, this.height);
          that.objectsLoading = true;
          that.loadPageObjects();
        };

        image.src = this.get("imageUrl");
      }
      else {
        this.image = null;
      }
    },

    loadPageObjects: function () {
      this.rectangles = {};

      var pageId = this.get("pageId");
      if (pageId != "") {
        var self = this;
        applib.getItemFromDatabase(pageId, function (page) {
          self.pageWidth = self.convertMillimetersToPixels(page["Width"]);
          var scale = self.viewModel.$el[0].width / self.pageWidth;

          applib.getEditorTypesQuery(function (editorTypesQuery) {
            var query = "//*[@@id = '" + pageId + "']//*[" + editorTypesQuery + "]";
            var options = { language: applib.contentLanguage, fields: ["x", "y", "width", "height"] };

            applib.database.query(query, function (items) {
              _.each(items, function (item) {
                self.createRectangle(item, scale);
              }, self);

              self.set("items", items);
              self.objectsLoading = false;

            }, options);
          }, self);
        }, self);
      }
    },

    reloadPageObjects: function () {
      if (!this.objectsLoading) {
        this.rectangles = {};
        if (this.pageWidth != "") {
          var scale = this.viewModel.$el[0].width / this.pageWidth;

          var items = this.get("items");
          var self = this;
          _.each(items, function (item) {
            self.createRectangle(item, scale);
          }, self);
        }
      }
    },

    createRectangle: function (item, scale) {
      var originalX = item["X"];
      var originalY = item["Y"];
      var originalW = item["Width"];
      var originalH = item["Height"];

      var w = scale * this.convertMillimetersToPixels(originalW);
      var h = scale * this.convertMillimetersToPixels(originalH);

      var x = scale * this.convertCoordinates(this.convertMillimetersToPixels(originalX), this.convertMillimetersToPixels(originalW));
      var y = scale * this.convertCoordinates(this.convertMillimetersToPixels(originalY), this.convertMillimetersToPixels(originalH));
      var rectangle = { x: x, y: y, width: w, height: h };
      this.rectangles[item.itemId] = rectangle;
      return rectangle;
    },

    highlightItems: function (items) {
      if (this.image != null) {
        var canvas = this.viewModel.$el[0];
        var context = canvas.getContext('2d');
        context.clearRect(0, 0, canvas.width, canvas.height);
        context.beginPath();
        context.drawImage(this.image, 0, 0, canvas.width, canvas.height);
        if (items != null) {
          _.each(items, function (item) {
            this.drawItem(item, context);
          }, this);
        }
      }
    },

    highlightItem: function () {
      var item = this.get("item");
      var isSnippet = (item != null && item.$templateName == "P_Snippet");
      if (this.image != null && !isSnippet) {
        var canvas = this.viewModel.$el[0];
        var context = canvas.getContext('2d');
        context.clearRect(0, 0, canvas.width, canvas.height);
        context.beginPath();
        context.drawImage(this.image, 0, 0, canvas.width, canvas.height);

        if (item != null) {
          this.drawItem(item, context);
        }
      }
    },

    highlightRectangle: function (rectangle) {
      if (this.image != null) {
        var canvas = this.viewModel.$el[0];
        var context = canvas.getContext('2d');
        context.clearRect(0, 0, canvas.width, canvas.height);
        context.beginPath();
        context.drawImage(this.image, 0, 0, canvas.width, canvas.height);

        this.drawRectangle(rectangle, context);
      }
    },

    drawItem: function (item, context) {
      var rectangle = this.rectangles[item.itemId];
      if (rectangle == null) {
        if (this.pageWidth != "") {
          var pageWidth = this.convertMillimetersToPixels(this.pageWidth);
          var scale = context.canvas.width / pageWidth;

          rectangle = this.createRectangle(item, scale);
        }
      }

      this.drawRectangle(rectangle, context);
    },

    drawRectangle: function (rectangle, context) {
      if (rectangle != null) {
        context.fillStyle = 'rgba(0,255,255,0.5)';
        context.strokeStyle = 'rgba(255,0,0,0.7)';
        context.fillRect(rectangle["x"], rectangle["y"], rectangle["width"], rectangle["height"]);
        context.strokeRect(rectangle["x"], rectangle["y"], rectangle["width"], rectangle["height"]);
      }
    },

    convertCoordinates: function (position, size) {
      return position - size / 2;
    },

    convertMillimetersToPixels: function (value) {
      var num = parseFloat(value);
      return num * 3.78;
    }
  });

  var view = _sc.Definitions.Views.ControlView.extend({
    initialize: function (options) {
      this._super();
    }
  });

  _sc.Factories.createComponent("CanvasComponent", model, view, ".sc-CanvasComponent");
});