﻿/// <reference path="../../../../../../assets/lib/dist/sitecore.js" />

define(["sitecore", "/-/speak/v1/odg/applib.js"], function (_sc, applib) {
  var editDesign = _sc.Definitions.App.extend({
    initialized: function () {
      _sc.on("assignDataDialog", function (editContext) {
        this.openAssignDialog(editContext, applib.settings.renderings.assignDataDialog, "assignDataDialog", "openAssignDataDialog");
      }, this);

      _sc.on("assignImageDialog", function (editContext) {
        this.openAssignDialog(editContext, applib.settings.renderings.assignImageDialog, "assignImageDialog", "openAssignImageDialog");
      }, this);

      _sc.on("openPageItemEdit", this.openPageItemEdit, this);
      _sc.on("snippetUpdated", this.snippetUpdated, this);
      _sc.on("pageItemUpdated", this.pageItemUpdated, this);

      applib.contentLanguage = this.LanguageSwitcher.get("contentLanguage");
      this.PagesListControl.set("contentLanguage", applib.contentLanguage);
      this.LanguageSwitcher.on("change:contentLanguage", this.languageChanged, this);

      this.PagesListControl.on("change:selectedItemId", this.setActivePage, this);

      this.PagesDataSource.on("change:language", this.PagesDataSource.refresh, this.PagesDataSource);
      this.PagesDataSource.on("change:items", this.updatePagesListControl, this);

      this.PageItemEditSmartPanel.on("change:isOpen", this.unfocusItem, this);
    },

    snippetUpdated: function () {
      this.PageItemEditSmartPanel.set("isOpen", false);
      this.MessageBar.set("isVisible", true);
      this.forceSilentTaskUpdate();
    },

    pageItemUpdated: function () {
      this.MessageBar.set("isVisible", true);
      this.PageItemEditSmartPanel.set("isOpen", false);
      this.Canvas.loadPageObjects();
      this.forceSilentTaskUpdate();
    },

    forceSilentTaskUpdate: function () {
      var url = this.item.toModel().getUrl();

      var ajaxOptions = {
        dataType: "json",
        type: "PUT",
        data: [{ name: "__Updated", value: _sc.Helpers.date.toISO(new Date()) }]
      };

      $.ajax(url, ajaxOptions);
    },

    openAssignDialog: function (editContext, dialogId, dialogName, eventName) {
      if (!this[dialogName]) {
        var app = this;
        applib.addDialog(this, dialogId, dialogName, function () {
          _sc.trigger(eventName, app.item, editContext);
        }, this);
        return;
      }

      _sc.trigger(eventName, this.item, editContext);
    },

    unfocusItem: function () {
      if (!this.PageItemEditSmartPanel.get("isOpen")) {
        this.Canvas.set("item", null);
        this.PageItemEditComponent.set("item", null);
        _sc.trigger("clearSelection", "");
      }
    },

    languageChanged: function () {
      applib.contentLanguage = this.LanguageSwitcher.get("contentLanguage");
      this.PagesListControl.set("contentLanguage", applib.contentLanguage);
      this.loadEditor(this.item, this.PagesListControl.get("selectedItemId"));
    },

    loadEditor: function (item, selectedPageId) {
      if (!item || item.$templateName != "Design Document") {
        return;
      }

      this.item = item;

      this.DesignEditComponent.prepare();
      this.PagesListControl.set("defaultSelectedItemId", selectedPageId);
      this.PagesListControl.set("items", []);
      this.PagesListControl.set("selectedItem", null);
      this.PagesListControl.set("selectedItemId", "");
      this.MessageBar.set("isVisible", false);
      this.Canvas.set("imageUrl", "");
      this.Canvas.set("items", null);

      this.PagesDataSource.set("language", applib.contentLanguage);
      this.PagesDataSource.set("database", item.$database);
      this.PagesDataSource.set("query", "//*[@@id = '" + item["Local Project"] + "']//*[@@templatekey='p_page']");
    },

    openPageItemEdit: function (item, snippetEditControl) {
      if (item != null) {
        this.PageItemEditComponent.set("item", item);
        this.PageItemEditComponent.snippetEditControl = snippetEditControl;
        this.Canvas.set("item", item);
        this.PageItemEditSmartPanel.set("isOpen", true);
        this.PageItemEditHeaderTitle.set("text", item.$displayName);

        _sc.trigger("clearSelection", item.itemId);
      }
    },

    saveItemChanges: function () {
      var self = this;
      this.PageItemEditComponent.saveChanges(function (hasChanges) {
        if (!hasChanges) {
          self.PageItemEditSmartPanel.set("isOpen", false);
        }
      });
    },

    cancelEdit: function () {
      this.PageItemEditSmartPanel.set("isOpen", false);
    },

    updatePagesListControl: function () {
      var selectedPageId = this.PagesListControl.get("defaultSelectedItemId");
      var items = this.PagesDataSource.get("items");
      if (!selectedPageId && items.length > 0) {
        this.PagesListControl.set("defaultSelectedItemId", items[0].itemId);
      }
      this.PagesListControl.set("items", items);
    },

    setActivePage: function () {
      this.MessageBar.set("isVisible", false);
      this.PageItemEditSmartPanel.set("isOpen", false);
      var selectedItem = this.PagesListControl.get("selectedItem");

      if (!selectedItem) {
        this.Canvas.set("imageUrl", "");
        this.Canvas.set("items", null);
        return;
      }

      var pageMediaUrl = selectedItem.get("pagemediaurl");
      if (pageMediaUrl && pageMediaUrl.length > 0) {
        this.Canvas.set("imageUrl", pageMediaUrl);
      }
      else {
        this.getPreviewImage(selectedItem, false);
      }
    },

    generatePreview: function () {
      var selectedItem = this.PagesListControl.get("selectedItem");
      if (selectedItem) {
        this.PageItemEditSmartPanel.set("isOpen", false);
        this.MessageBar.set("isVisible", false);
        this.getPreviewImage(selectedItem, true);
      }
    },

    getPreviewImage: function (selectedItem, forceNew) {
      var that = this;

      var serverRequestOptions = {};
      serverRequestOptions["url"] = applib.settings.navigation.commandRoutePrefix + "design/getpreviewimage";
      serverRequestOptions["onSuccess"] = function (data) {
        selectedItem.set("pagemediaurl", data.mediaUrl);
        that.Canvas.set("imageUrl", data.mediaUrl);

        if (data.thumbnailUrl && data.thumbnailUrl.length > 0) {
          selectedItem.set("$mediaurl", data.thumbnailUrl);
          selectedItem.set("image", data.thumbnailUrl);
        }
      };

      serverRequestOptions.parameters = {
        "itemId": selectedItem.get("itemId"),
        lang: applib.contentLanguage,
        forceNew: forceNew,
        useHighRes: false,
        saveInPage: true
      };

      this.GenericDataProvider.viewModel.getData(serverRequestOptions);
    }
  });

  return editDesign;
});