﻿define(["sitecore", "/-/speak/v1/odg/applib.js"], function (_sc, applib) {
  var snippetEdit = _sc.Definitions.App.extend({
    initialized: function () {
      this.set("itemReferenceId", "");
      this.on("change:itemReferenceId", this.loadReference, this);

      this.set("item", null);
      this.on("change:item", this.loadSnippet, this);

      this.snippetObjects = null;
      this.on("change:snippetItems", this);

      this.SnippetObjectsListControl.on("change:selectedItemId", this.selectionChanged, this);
      _sc.on("clearSelection", this.clearSelection, this);

      this.Border.viewModel.$el.addClass("edit-design");
    },

    loadSnippet: function () {
      var item = this.get("item");
      if (item != null) {
        this.SnippetAccordion.set("header", item.$displayName);
      }
      else {
        this.Border.set("isVisible", false);
      }
      this.reloadData();
    },

    loadReference: function () {
      var referenceId = this.get("itemReferenceId");
      var item = this.get("item");
      var self = this;
      $.post(applib.settings.navigation.commandRoutePrefix + "design/assigndatatosnippet",
      {
        "snippetId": item.itemId,
        "dataId": referenceId,
        "lang": applib.contentLanguage
      }).done(function (data) {
        self.reloadData();
        _sc.trigger("snippetUpdated");
      }).fail(function (err) {
      });
    },

    reloadData: function () {
      var item = this.get("item");
      this.snippetObjects = null;

      if (item != null) {
        var self = this;

        applib.getEditorTypesQuery(function (editorTypesQuery) {
          var query = "//*[@@id = '" + item.itemId + "']//*[" + editorTypesQuery + "]";
          var options = { language: applib.contentLanguage, payLoad: "full" };

          applib.database.query(query, function (items) {
            self.snippetObjects = items;
            self.SnippetObjectsListControl.set("items", items);
          }, options);
        }, this);
      }
      else {
        this.SnippetObjectsListControl.set("items", []);
      }
    },

    selectionChanged: function () {
      var selectedItemId = this.SnippetObjectsListControl.get("selectedItemId");
      if (selectedItemId && selectedItemId.length > 0) {
        var self = this;
        applib.getItemFromList(this.SnippetObjectsListControl, function (data) {
          _sc.trigger("openPageItemEdit", data, self);
        }, this);
      }
    },

    clearSelection: function (itemId) {
      var selectedItemId = this.SnippetObjectsListControl.get("selectedItemId");
      if (selectedItemId && selectedItemId.length > 0 && itemId != selectedItemId) {
        this.SnippetObjectsListControl.set("selectedItemId", null);
      }
    },

    assignDataDialog: function () {
      _sc.trigger("assignDataDialog", this);
    }
  });

  return snippetEdit;
});