﻿define(["sitecore", "/-/speak/v1/odg/applib.js"], function (_sc, applib) {
  var textEdit = _sc.Definitions.App.extend({
    initialized: function () {
      this.contentItem = null;
      this.set("item", null);
      this.on("change:item", this.loadItem, this);
      this.FieldValueTextBox.on("change:text", this.changeText, this);
    },

    changeText: function () {
      if (!this.isValid()) {
        $("[data-sc-id=FieldValueTextBox]").css("border-color", "red");
      }
      else {
        $("[data-sc-id=FieldValueTextBox]").css("border-color", "");
      }
    },

    loadItem: function () {
      var item = this.get("item");
      if (item != null) {
        var self = this;
        applib.getChildren(item.itemId, function (items, total, data) {
          if (total != null && total > 0) {
            self.contentItem = items[0].toModel();
            self.originalText = self.cleanXml(items[0]["Content"]);
            self.FieldValueTextBox.set("text", self.originalText);
          }
          else {
            self.originalText = "";
            self.FieldValueTextBox.set("isEnabled", false);
            self.FieldValueTextBox.viewModel.$el[0].title = "";
            self.FieldValueTextBox.viewModel.$el[0].placeholder = "";
          }
        }, true);
      }
    },

    cleanXml: function (originalString) {
      originalString = originalString.replace(/<!\[CDATA\[\s*|\s*\]\]>/g, "");
      return originalString.replace(/(<([^>]+)>)/ig, "");
    },

    acceptChanges: function () {
      if (this.hasChanges()) {
        if (this.contentItem != null) {
          this.contentItem.set("Content", this.FieldValueTextBox.get("text"));
          this.contentItem.getUrl = applib.getUrl;
          this.contentItem.save();
          this.originalText = this.FieldValueTextBox.get("text");
        }

        return true;
      }

      return false;
    },

    hasChanges: function () {
      return this.originalText != this.FieldValueTextBox.get("text");
    },

    isValid: function () {
      var regEx = /(<([^>]+)>)/ig;
      return !(regEx.test(this.FieldValueTextBox.get("text")));
    }
  });

  return textEdit;
});