﻿define(["sitecore", "/-/speak/v1/odg/applib.js"], function (_sc, applib) {
  var model = _sc.Definitions.Models.ControlModel.extend({
    initialize: function (options) {
      this._super();

      this.confirmationText = "";
      this.prepare();

      this.set("pageId", "");
      this.on("change:pageId", this.initPageObjects, this);
    },

    initPageObjects: function () {

      var pageId = this.get("pageId");

      if (pageId != this.previousPageId) {
        this.cleanControls();
        this.previousPageId = pageId;

        if (pageId != "") {
          var shortId = _sc.Helpers.id.toShortId(pageId);
          if (this["DesignEdit" + shortId] == null) {
            var self = this;
            
            var query = "//*[@@id = '" + pageId + "']//*[@@templatekey='p_snippet']";
            var options = { language: applib.contentLanguage, payLoad: "full" };
            
            applib.database.query(query, function (items) {
              self.addDesignEdit(shortId, items, self.viewModel.$el, self);
            }, options);
          }
        }
      }
    },

    prepare: function () {
      this.designControlId = "";
      this.previousPageId = "";
      this.designControls = new Array();
    },

    cleanControls: function () {
      if (this.viewModel.app[this.designControlId] != null) {
        this.viewModel.app[this.designControlId].destroy();
      }

      this.viewModel.$el.empty();
      this.prepare();
    },

    addDesignEdit: function (itemId, items, container, self) {
      var temp = this.viewModel.app;

      if (self.designControlId != "DesignEdit" + itemId) {
        self.designControlId = "DesignEdit" + itemId;
        this.viewModel.app.insertRendering(applib.settings.renderings.designEdit, { path: applib.settings.renderings.renderingRootPath, $el: container }, function (designEdit) {
          temp["DesignEdit" + itemId] = designEdit;
          self.addItem(designEdit, 0, items, self);
        });
      }
    },

    addItem: function (designEdit, idx, items, self) {
      if (idx < items.length) {
        if (designEdit.Border != null) {
          self.addItemEdit(items[idx], designEdit.Border.viewModel.$el, designEdit, function () {
            self.addItem(designEdit, ++idx, items, self);
          });
        }
      }
    },

    addItemEdit: function (item, container, designEdit, renderNext) {

      var controlId = "SnippetEdit" + _sc.Helpers.id.toShortId(item.itemId);

      if (this.designControls[controlId] == null) {
        var self = this;
        this.viewModel.app.insertRendering(applib.settings.renderings.snippetEdit, { path: applib.settings.renderings.renderingRootPath, $el: container }, function (control) {
          if (control != null) {
            control.set("item", item);
            designEdit[controlId] = control;
            self.designControls.push(controlId);
            renderNext();
          }
        }
        );
      }
    }
  });

  var view = _sc.Definitions.Views.ControlView.extend({
    initialize: function (options) {
      this._super();
    }
  });

  _sc.Factories.createComponent("DesignEditComponent", model, view, ".sc-DesignEditComponent");
});
