﻿define(["sitecore", "/-/speak/v1/odg/applib.js"], function (_sc, applib) {
  var assignDataDialog = _sc.Definitions.App.extend({
    initialized: function () {
      _sc.on("openAssignDataDialog", this.openDialog, this);

      this.set("dataSources", "");
      this.on("change:dataSources", this.loadDataSources, this);

      this.InsertButton.set("isEnabled", false);
      this.DataDialog.viewModel.$el.addClass("frameDialog");
    },

    openDialog: function (item, dataEditContext) {
      if (!item || item.$templateName != "Design Document") {
        return;
      }
      this.dataEditContext = dataEditContext;

      if (item["Design Collection"] && item["Design Collection"].length > 0) {
        var app = this;
        applib.getItemFromDatabase(item["Design Collection"], function (dataItem) {
          app.set("dataSources", dataItem != null ? dataItem["Data Sources"] : "");
        });
      }
      else {
        this.set("dataSources", "");
      }

      this.DataDialog.show();
    },

    loadDataSources: function () {
      var dataSourceIds = this.get("dataSources");
      if (!dataSourceIds || dataSourceIds.length == 0) {
        return;
      }

      var self = this;
      var ajaxOptions = {
        url: applib.settings.navigation.commandRoutePrefix + "collection/filterdatasources",
        type: "GET",
        dataType: "json",
        data: {
          itemIds: dataSourceIds
        }
      };

      $.ajax(ajaxOptions).done(function (data) {
        _.each(data, function (itemId) {
          self.addDataTree(itemId, self.DataTreeBorder.viewModel.$el, self);
        }, self);
      }).fail(function (err) {
      });
    },

    addDataTree: function (itemId, container, app) {
      var temp = app;
      var contorlId = "DataTree" + _sc.Helpers.id.toShortId(itemId);
      var itemPlusRootId = applib.settings.renderings.dataTreeView + "&ro=" + itemId;

      app.insertRendering(itemPlusRootId, { path: applib.settings.renderings.renderingRootPath, $el: container }, function (control) {
        control.TreeView.on("change:selectedItemId", function () {
          app.selectionChanged(control);
        }, temp);
        temp[contorlId] = control;
      });
    },

    selectionChanged: function (control) {
      var selectedNode = control.TreeView.viewModel.getTree().activeNode;
      if (selectedNode) {
        this.setSelectedItem(selectedNode);

        if (!this.prevControl) {
          this.prevControl = control;
        }

        // deselect and deactivate previous selected node if coming from different treeview
        if (this.prevControl.name !== control.name) {
          var otherSelectedNode = this.prevControl.TreeView.viewModel.getTree().activeNode;
          if (otherSelectedNode) {
            otherSelectedNode.deactivate();
            this.prevControl.TreeView.set("selectedNode", null);
            this.prevControl.TreeView.set("selectedItemId", null);
            this.prevControl.TreeView.set("selectedItemPath", null);
          }
          this.prevControl = control;
        }
      }
    },

    setSelectedItem: function (selectedNode) {
      if (selectedNode.data && selectedNode.data.rawItem == undefined) {
        var self = this;
        applib.database.getItem(selectedNode.data.itemUri, function (data) {
          selectedNode.data.rawItem = data;
          self.loadSelectedItem(data);
        });
      }

      this.loadSelectedItem(selectedNode.data.rawItem || selectedNode.data);
    },

    loadSelectedItem: function (selItem) {
      this.selectedItem = selItem;
      this.SelectedPathText.set("text", this.selectedItem ? this.selectedItem.$path || this.selectedItem.path : "");
      this.InsertButton.set("isEnabled", this.selectedItem ? true : false);

    },

    closePopup: function () {
      this.DataDialog.hide();
    },

    proceedSelection: function () {
      this.closeDialog(this.selectedItem);

      if (this.selectedItem && this.selectedItem.itemId && this.dataEditContext) {
        this.dataEditContext.set("itemReferenceId", this.selectedItem.itemId);
      }

      this.closePopup();
    },

    close: function () {
      this.closeDialog(null);
      this.closePopup();
    }
  });

  return assignDataDialog;
});