﻿/// <reference path="../../../../../../assets/lib/dist/sitecore.js" />

define(["sitecore", "/-/speak/v1/odg/applib.js"], function (_sc, applib) {
  return _sc.Definitions.App.extend({
    initialized: function () {
      _sc.on("createDialog", this.openDialog, this);
      this.NameTextBoxElem().context = this;
      this.NameTextBoxElem().onkeyup = this.enableCreateButton;
      this.isSaving = false;

      this.CollectionDialog.viewModel.$el.addClass("frameDialog");
    },

    NameTextBoxElem: function () {
      return this.NameTextBox.viewModel.$el[0];
    },

    enableCreateButton: function (e) {
      var isEnabled = this.context.NameTextBoxElem().value.length > 0;

      this.context.CreateButton.set("isEnabled", isEnabled);
      if (e.keyCode == 13 && isEnabled) {
        this.context.CreateButton.viewModel.click();
      }
    },

    openDialog: function () {
      this.NameTextBox.set("text", "");
      $("[data-sc-id=NameTextBox]").css("border-color", "");
      this.CreateButton.set("isEnabled", false);
      this.MessageBar.removeMessages();
      this.CollectionDialog.show();
      
      var nextElem = this.ContentBorder.viewModel.$el.next();
      if (nextElem.hasClass("sc-dialogWindow-buttons")) {
        this.ContentBorder.viewModel.$el.css("margin-bottom", nextElem.outerHeight(true));
      }
    },

    closePopup: function () {
      this.CollectionDialog.hide();
    },

    /* create with item wep api - works but if any error - returns undefined*/
    createCollection: function () {
      if (this.isSaving) {
        return;
      }

      this.MessageBar.removeMessages();
      var errorMessage;
      var name = this.NameTextBox.get("text");
      var isValid = name && name.length > 0;
      $("[data-sc-id=NameTextBox]").css("border-color", isValid ? "" : "red");
      if (!isValid) {
        errorMessage = this.StringDictionary.get("An item name cannot be blank.");
        if (!errorMessage || errorMessage.length == 0) {
          errorMessage = this.StringDictionary.translate("'$Input' is not a valid name.");
          errorMessage = errorMessage.replace("$Input", "");
        }
        this.MessageBar.addMessage('error', errorMessage);
        return;
      }

      var self = this;
      this.isSaving = true;
      this.ProgressIndicator.set("isBusy", true);

      _sc.Definitions.Data.createItem({
        templateId: applib.settings.templateIds.designCollection,
        name: name,
        parentId: applib.settings.rootFolderIds.designCollections,
        database: applib.settings.masterDatabaseName
      }, {
        "__Display name": name
      }, function (item) {
        self.goToCollection(item.itemId);
      }).fail(function (response) {
        self.isSaving = false;
        self.ProgressIndicator.set("isBusy", false);

        errorMessage = self.StringDictionary.get(response.responseText);
        if (!errorMessage || errorMessage.length == 0) {
          errorMessage = self.StringDictionary.translate("'$Input' is not a valid name.");
          errorMessage = errorMessage.replace("$Input", name);
        }
        self.MessageBar.addMessage('error', errorMessage);
      });
    },

    goToCollection: function (id) {
      window.location = applib.settings.navigation.url + "editcollection?id=" + id + "&ref=DesignManager";
    }
  });
});
