﻿define(["sitecore", "/-/speak/v1/odg/applib.js"], function (_sc, applib) {
  var model = _sc.Definitions.Models.ControlModel.extend({
    initialize: function (options) {
      this._super();

      this.confirmationText = "";
      this.editControlId = "";
      this.snippetEditControl = null;
      this.set("item", null);
      this.on("change:item", this.initPageItem, this);
      _sc.on("item:updated", this.itemUpdated, this);
    },

    initPageItem: function () {
      this.clearEdit();

      var item = this.get("item");
      if (item != null) {
        this.editItem = _.clone(item);
        this.addItemEdit(this.editItem, this.viewModel.$el);
      }
    },

    clearEdit: function () {
      if (this.attributesControls != null) {
        _.each(this.attributesControls, function (ctrl) {
          ctrl.destroy();
        }, this);
      }

      this.editItem = null;
      this.attributesControls = null;
      this.viewModel.$el.empty();
    },

    addItemEdit: function (item, container) {
      var renderingId;
      var fieldName = item["Item Field"];
      this.attributesControls = new Array();
      var self = this;
      applib.getEditorType(item.$templateName, function (data) {
        if (data != null) {
          if (fieldName == null || fieldName == "") {
            renderingId = data.renderingId;
          }
          else {
            renderingId = data.dataRenderingId;
          }

          if (renderingId != "") {

            self.viewModel.app.insertRendering(renderingId, { path: applib.settings.renderings.renderingRootPath, $el: container }, function (control) {
              if (control != null) {
                  control.set("item", item);
                  self.attributesControls.push(control);
              }

              self.addAttributesControls(data.additionalRenderings, container, item);
            });
          }
          else {
            self.addAttributesControls(data.additionalRenderings, container, item);
          }
        }
      });
    },

    addAttributesControls: function (additionalRenderings, container, item) {
      if (additionalRenderings != null && additionalRenderings.length > 0 && additionalRenderings != "") {
        var renderings = additionalRenderings.split('|');
        this.addControls(container, 0, renderings, item);
      }
    },

    addControls: function (container, idx, renderings, dataItem) {
      if (idx < renderings.length) {
        var self = this;
        this.addEditControl(renderings[idx], container, dataItem, function () {
          self.addControls(container, ++idx, renderings, dataItem);
        });
      }
    },

    addEditControl: function (renderingId, container, dataItem, renderNext) {
      var self = this;
      this.viewModel.app.insertRendering(renderingId, { path: applib.settings.renderings.renderingRootPath, $el: container }, function (control) {
        if (control != null) {
          control.set("item", dataItem);
          self.attributesControls.push(control);
          renderNext();
        }
      }
      );
    },

    saveChanges: function (changed) {
      if (this.hasChanges()) {
        if (this.isValid()) {
          this.isSaving = true;
          this.viewModel.app.ProgressIndicator.set("isBusy", true);
          var editItemModel = this.editItem.toModel();
          editItemModel.getUrl = applib.getUrl;
          editItemModel.save();
          changed(true);
        }
      }
      else {
        changed(false);
      }
    },

    itemUpdated: function (value) {
      var item = this.get("item");
      if (item != null && applib.isUpdatedItem(item.itemId, value.url)) {
        var self = this;
        this.acceptChanges(function () {
          self.isSaving = false;
          self.viewModel.app.ProgressIndicator.set("isBusy", false);

          if (self.snippetEditControl != null) {
            self.snippetEditControl.reloadData();
          }

          _sc.trigger("pageItemUpdated");
        });
      }
    },

    isValid: function () {
      var isValid = true;

      _.each(this.attributesControls, function (ctrl) {
        if (isValid) {
          isValid = ctrl.isValid();
        }
      }, this);

      return isValid;
    },

    hasChanges: function () {
      var hasChanges = false;

      _.each(this.attributesControls, function (ctrl) {
        if (!hasChanges) {
          hasChanges = ctrl.hasChanges();
        }
      }, this);

      return hasChanges;
    },

    acceptChanges: function (changesAccepted) {
      _.each(this.attributesControls, function (ctrl) {

        if (ctrl.hasChanges()) {
          ctrl.acceptChanges();
        }

      }, this);

      changesAccepted();
    }
  });


  var view = _sc.Definitions.Views.ControlView.extend({
    initialize: function (options) {
      this._super();
    }
  });

  _sc.Factories.createComponent("PageItemEditComponent", model, view, ".sc-PageItemEditComponent");

});