﻿define(["sitecore", "/-/speak/v1/odg/applib.js"], function (Sitecore, applib) {
  "use strict";

  var searchDataSource = _.find(Sitecore.Components, function (comp) { return comp.type === "SearchDataSource"; });

  var model = searchDataSource.model.extend(
    {
      initialize: function (attributes) {
        this._super();
        this.set("itemId", null);
        this.set("listField", null);
      },

      getItems: function () {
        if (!this.isReady) {
          return;
        }

        var search = this.get("text"),
        options = this.getOptions(),
        url,
        selectedFacets;

        if (!search && !options.root && !options.searchConfig) {
          return;
        }

        url = search || "";

        selectedFacets = this.get("selectedFacets");
        if (selectedFacets != null && selectedFacets.length > 0) {
          url = this.getFacets(url, selectedFacets);
        }

        var databaseUri = new Sitecore.Definitions.Data.DatabaseUri(this.get("database"));
        var database = new Sitecore.Definitions.Data.Database(databaseUri);

        this.pendingRequests++;
        this.set("isBusy", true);

        Sitecore.debug("SearchDataSource request: '", url, "', options:", options);

        // ODG related change
        applib.search(database, url, $.proxy(this.completed, this), options);
      },

      getOptions: function () {
        var options = this._super();

        options.itemId = this.get("itemId");
        options.listField = this.get("listField");

        return options;
      }
    }
  );

  var view = searchDataSource.view.extend(
    {
      initialize: function (options) {
        this.model.set("itemId", this.$el.attr("data-sc-item-id"));
        this.model.set("listField", this.$el.attr("data-sc-list-field"));

        this._super();
      }
    }
  );

  Sitecore.Factories.createComponent("FieldSearchDataSource", model, view, "script[type='text/x-sitecore-fieldsearchdatasource']");
});