﻿define(["sitecore", "/-/speak/v1/odg/applib.js"], function (_sc, applib) {
  return _sc.Definitions.App.extend({
    initialized: function () {
      $.ajaxSetup({ cache: false });

      _sc.on("view:task", this.editTask, this);

      this.TasksListControl.on("change:selectedItemId", this.setInfoPanel, this);
    },

    setNavigationId: function (guid) {
      var isGuid = guid && _sc.Helpers.id.isId(guid);
      if (isGuid) {
        if (this.NavigationGroup.get("selectedItemId") == guid) {
          this.NavigationGroup.set("selectedItemId", "");
        }
        this.NavigationGroup.set("selectedItemId", guid);
      }

      return isGuid;
    },

    setInfoPanel: function () {
      var selected = this.TasksListControl.get("selectedItemId");
      if (selected && selected.length > 0) {
        applib.getItemFromList(this.TasksListControl, this.fillItemInfo, this);
      }
    },

    fillItemInfo: function (item) {
      this.ItemInfoContent.set("item", item);
    },

    editTask: function (itemId) {
      if (arguments.length == 0) {
        itemId = this.TasksListControl.get("selectedItemId");
      }
      if (itemId && itemId.length > 0) {
        window.location = applib.settings.navigation.url + "editdocument?id=" + itemId + "&ref=documents";
      }
    },

    deleteTask: function () {
      this.MessageBar.removeMessages();

      var selectedItems = this.TasksListControl.get("checkedItems");
      applib.getItemFromList(this.TasksListControl, function (data) {
        if (data != null) selectedItems = [data];
      }, this);
      if (!selectedItems || selectedItems.length == 0) {
        return;
      }

      var deleteConfirmationText;
      if (selectedItems.length == 1) {
        deleteConfirmationText = _sc.Helpers.string.format(this.StringDictionary.translate("Are you sure you want to delete \"{0}\"?"), selectedItems[0].$displayName);
      }
      else {
        deleteConfirmationText = _sc.Helpers.string.format(this.StringDictionary.translate("Are you sure you want to delete these {0} items?{1}"), selectedItems.length, "");
      }

      if (confirm(deleteConfirmationText)) {
        var totalCount = selectedItems.length;
        var noPermissionMessage = this.StringDictionary.translate("You do not have permission to delete \"{0}\".");
        var deleteItemIds = [];
        _.each(selectedItems, function (selItem) {
          applib.getItemAccess(selItem.itemId, function (data) {
            if (data.canDelete) {
              deleteItemIds.push(selItem.itemId);
            } else {
              this.MessageBar.addMessage("warning", _sc.Helpers.string.format(noPermissionMessage, selItem.$displayName));
            }
            if (!--totalCount) {
              this.doDeleteTasks(deleteItemIds);
            }
          }, this);
        }, this);
      }
    },

    doDeleteTasks: function (deleteItemIds) {
      var that = this;
      var deleteCount = deleteItemIds.length;

      var refreshDataSource = function (data) {
        /*
        if (data && data.statusCode !== 200 && data.error && data.error.message.length > 0) {
          that.MessageBar.addMessage("error", data.error.message);
        }
        */
        if (!--deleteCount) {
          that.TasksListControl.set("selectedItemId", null);
          that.DataSource.refresh();
        }
      };

      _.each(deleteItemIds, function (itemId) {
        applib.getItemFromDatabase(itemId, function (item) {
          if (item == null) {
            refreshDataSource.call(that, null);
          }

          var localProjectId = item["Local Project"];
          var designTemplateId = item["Design Template"];
          var collectionId = item['Design Collection'];

          var completed = function () {
            refreshDataSource.call(that, null);

            if (localProjectId && localProjectId.length > 0) {
              applib.getItemFromDatabase(localProjectId, function (projectItem) {
                if (projectItem != null) {
                  projectItem.toModel().remove();
                }
              });
            }

            if (collectionId && collectionId.length > 0) {
              $.ajax({
                url: applib.settings.navigation.commandRoutePrefix + "collection/updateusage",
                type: "PUT",
                dataType: "json",
                data: { itemId: collectionId }
              });
            }

            if (designTemplateId && designTemplateId.length > 0) {
              $.ajax({
                url: applib.settings.navigation.commandRoutePrefix + "design/updateusage",
                type: "PUT",
                dataType: "json",
                data: { itemId: designTemplateId }
              });
            }
          };

          item.toModel().remove(completed, that).fail(refreshDataSource, that);
        }, this);
      }, this);
    },

    newTask: function () {
      var self = this;

      var itemId = this.TasksListControl.get("selectedItemId");
      if (itemId && itemId.length > 0) {
        applib.getItemFromDatabase(itemId, function (item) {
          if (item == null) {
            return;
          }

          var testCollectionMissing = function (collectionCondition) {
            if (!collectionCondition)
              alert(self.MissingCollectionLabel.get("text") || "Collection is not available");
            return !collectionCondition;
          };

          var collectionId = item["Design Collection"];

          if (testCollectionMissing(collectionId && collectionId.length > 0))
            return;

          applib.getCollectionState(collectionId, function (result) {
            if (testCollectionMissing(result.isExisting && result.canRead))
              return;
            
              if (result.isActive) {
                window.location = applib.settings.navigation.url + "adddocument?cid=" + collectionId + "&ref=documents";
              }
              else {
                alert(self.CollectionNotRunningText.get("text"));
              }
            });
        });
      }
    }
  });
});

