﻿define(["sitecore", "/-/speak/v1/odg/applib.js"], function (_sc, applib) {
  var editCollection = _sc.Definitions.App.extend({
    initialized: function () {
      $.ajaxSetup({ cache: false });
      var app = this;

      _sc.on("item:updated", this.itemUpdated, this);
      this.on("item:save", this.saveCollection, this);

      this.EndDateDatePicker.on("change:date", this.updateStartDateRequired, this);

      this.designs = new window.Backbone.Collection();
      this.designs.on("add", this.updateDesignsListControl, this);
      this.designs.on("remove", this.updateDesignsListControl, this);

      this.dataSources = new window.Backbone.Collection();
      this.dataSources.on("add", this.updateDataListControl, this);
      this.dataSources.on("remove", this.updateDataListControl, this);

      this.users = new window.Backbone.Collection();
      this.users.on("add", this.updateUsersListControl, this);
      this.users.on("remove", this.updateUsersListControl, this);

      this.roles = new window.Backbone.Collection();
      this.roles.on("add", this.updateRolesListControl, this);
      this.roles.on("remove", this.updateRolesListControl, this);

      var guid = applib.getUrlParameter("id");
      if (guid != "") {
        applib.getItemFromDatabase(guid, loadItem);
        applib.getItemAccess(guid, function (data) {
          this.SaveButton.set("isEnabled", data.canWrite);
        }, this);
      } else {
        this.back();
      }

      function loadItem(item) {
        if (item == null) {
          app.back();
        }

        app.item = item;
        app.itemModel = item.toModel();
        app.updateInfo();
      }

      this.SelectedDesignsListControl.on("change:selectedItemId", function () {
        this.setInfoPanel(this.SelectedDesignsListControl, this.DesignsListControl);
      }, this);
      this.DesignsListControl.on("change:selectedItemId", function () {
        this.setInfoPanel(this.DesignsListControl, this.SelectedDesignsListControl);
      }, this);

      this.DataBorder.viewModel.$el.addClass("selectdata-border");
      this.ButtonsBorder.viewModel.$el.addClass("vertical-buttons");
      this.RolesDialogWindow.viewModel.$el.addClass("frameDialog noFooter");
      this.UsersDialogWindow.viewModel.$el.addClass("frameDialog noFooter");
    },

    updateInfo: function () {
      if (!this.item || this.item.$templateName != "Design Collection") {
        return;
      }

      this.ItemInfoContent.set("item", this.item);

      this.NameTextBox.set("text", this.item.$displayName);

      this.InfoTextBox.set("text", this.item.Description);
      this.setDate(this.item["Start date"], this.StartDateDatePicker, this.StartDateTimePicker);
      this.setDate(this.item["End date"], this.EndDateDatePicker, this.EndDateTimePicker);

      applib.setItemsListValue(this.item['Design Templates'], this.designs);
      applib.setItemsListValue(this.item["Data Sources"], this.dataSources);
      this.updateAccountsLists();
    },

    setInfoPanel: function (currentListControl, otherListControl) {
      var selected = currentListControl.get("selectedItemId");
      var otherSelected = otherListControl.get("selectedItemId");

      var isSelected = selected && selected.length > 0;
      var isOtherSelected = otherSelected && otherSelected.length > 0;

      this.DesignInfoSmartPanel.set("isOpen", isSelected || isOtherSelected);

      if (isSelected) {
        if (isOtherSelected) {
          otherListControl.set("selectedItemId", null);
        }

        applib.getItemFromList(currentListControl, this.fillDesignInfo, this);
      }
    },

    fillDesignInfo: function (item) {
      this.DesignInfoContent.set("item", item);
    },

    saveCollection: function () {
      this.MessageBar.removeMessages();

      var name = this.NameTextBox.get("text");
      var startDate = this.getDate(this.StartDateDatePicker);
      var endDate = this.getDate(this.EndDateDatePicker);
      var isValid = true;

      if (!name || name.length == 0) {
        $("[data-sc-id=NameTextBox]").css("border-color", "red");
        $("[data-sc-id=NameTextBox1]").css("border-color", "red");
        isValid = false;
      }
      if (startDate == null && endDate != null) {
        $("[data-sc-id=StartDateDatePicker]").css("border-color", "red");
        $("[data-sc-id=StartDateTextBox1]").css("border-color", "red");
        this.MessageBar.addMessage('error', this.StartDateRequiredText.get("text"));
        isValid = false;
      }

      if (endDate && startDate > endDate) {
        this.MessageBar.addMessage('error', this.EndDateLessThanStartText.get("text"));
        isValid = false;
      }

      if (!isValid) {
        return;
      }

      $("[data-sc-id=StartDateDatePicker]").css("border-color", "");
      $("[data-sc-id=StartDateTextBox1]").css("border-color", "");

      this.itemModel.set("__Display name", name);
      this.itemModel.set("Name", name);
      this.itemModel.set("Description", this.InfoTextBox.get("text"));

      var self = this;
      this.getUtcDateValue(startDate, function (data) { self.itemModel.set("Start date", data); });
      this.getUtcDateValue(endDate, function (data) { self.itemModel.set("End date", data); });

      var designIds = this.designs.pluck("itemId");
      this.itemModel.set("Design Templates", designIds.join("|"));

      var dataSourceIds = this.dataSources.pluck("itemId");
      this.itemModel.set("Data Sources", dataSourceIds.join("|"));

      this.isSaving = true;
      this.ProgressIndicator.set("isBusy", true);

      this.itemModel.save();

      if (this.canAssignAccounts) {
        var userIds = this.users.pluck("itemId").join("|");
        var roleIds = this.roles.pluck("itemId").join("|");
        $.post(applib.settings.navigation.commandRoutePrefix + "collection/assignitemaccounts",
          {
            "itemId": this.item.itemId,
            "userIds": userIds,
            "roleIds": roleIds
          }).done(function (data) {
            // notify
          });
      }
    },

    itemUpdated: function (value) {
      if (this.item != null && applib.isUpdatedItem(this.item.itemId, value.url)) {
        this.isSaving = false;
        this.ProgressIndicator.set("isBusy", false);
        if (value.data && value.data.length > 0) {
          this.MessageBar.addMessage("notification", this.ChangesSavedText.get("text"));
        }
      }
    },

    updateStartDateRequired: function (datepicker, newDate) {
      var isRequired = newDate && newDate.length > 0;
      this.StartDateDatePicker.viewModel.$el[0].required = isRequired;
      if (!isRequired) {
        $("[data-sc-id=StartDateDatePicker]").css("border-color", "");
        $("[data-sc-id=StartDateTextBox1]").css("border-color", "");
      }
    },

    updateDesignsListControl: function () {
      this.SelectedDesignsListControl.set("items", this.designs.toJSON());
    },

    updateDataListControl: function () {
      this.SelectedDataListControl.set("items", this.dataSources.toJSON());
    },

    updateUsersListControl: function () {
      this.UsersListControl.set("items", this.users.toJSON());
    },

    updateRolesListControl: function () {
      this.RolesListControl.set("items", this.roles.toJSON());
    },

    updateAccountsLists: function () {
      var self = this;
      if (!this.item) {
        return;
      }

      $.get(applib.settings.navigation.commandRoutePrefix + "collection/getitemaccounts?itemId=" + this.item.itemId).done(function (data) {
        self.canAssignAccounts = data.changeable;
        self.UsersActionControl.set("isVisible", self.canAssignAccounts);
        self.RolesActionControl.set("isVisible", self.canAssignAccounts);
        self.users.add(data.users);
        self.roles.add(data.roles);
      });
    },

    addDesign: function () {
      var itemId = this.DesignsListControl.get("selectedItemId");
      if (!itemId) {
        return;
      }

      var test = this.designs.filter(function (item) {
        return item.get("itemId") == itemId;
      });
      if (test.length == 0) {
        var self = this;
        applib.getItemFromDatabase(itemId, function (dataItem) {
          self.designs.push(dataItem);
        });
      }
    },

    removeDesign: function () {
      var itemId = this.SelectedDesignsListControl.get("selectedItemId");
      if (!itemId) {
        return;
      }

      var test = this.designs.filter(function (item) {
        return item.get("itemId") == itemId;
      });
      if (test.length == 1) {
        this.designs.remove(test[0]);
        this.SelectedDesignsListControl.set("selectedItemId", null);
      }
    },

    openAddUsers: function () {
      this.UsersDialogWindow.show();
      var app = this;
      window.top.returnValue = null;

      var modalDialogClose = window.top.document.getElementById('jqueryModalDialogsFrame').contentWindow.dialogClose;
      var windowDialogClose = window.top.dialogClose;

      window.top.dialogClose = function () {
        if (window.top._scDialogs.length > 0) {
          modalDialogClose();
        }
        else {
          var result = window.top.returnValue;
          if (result && result.length > 0)
            app.addAccounts.call(app, result);
          app.UsersDialogWindow.hide();
        }
      };

      this.UsersDialogWindow.viewModel.$el.one("hide", function () {
        window.top.dialogClose = windowDialogClose;
      });
    },

    openAddRoles: function () {
      this.RolesDialogWindow.show();
      var app = this;
      window.top.returnValue = null;

      var modalDialogClose = window.top.document.getElementById('jqueryModalDialogsFrame').contentWindow.dialogClose;
      var windowDialogClose = window.top.dialogClose;

      window.top.dialogClose = function () {
        if (window.top._scDialogs.length > 0) {
          modalDialogClose();
        }
        else {
          var result = window.top.returnValue;
          if (result && result.length > 0)
            app.addAccounts.call(app, result);
          app.RolesDialogWindow.hide();
        }
      };

      this.RolesDialogWindow.viewModel.$el.one("hide", function () {
        window.top.dialogClose = windowDialogClose;
      });
    },

    addData: function () {
      var itemId = this.DataTreeView.get("selectedItemId");

      if (!itemId || itemId.length == 0) {
        return;
      }

      var test = this.dataSources.filter(function (item) {
        return item.get("itemId") == itemId;
      });
      if (test.length == 0) {
        var self = this;

        applib.getItemFromDatabase(itemId, function (dataItem) {
          self.dataSources.push(dataItem);
        });
      }
    },

    removeData: function () {
      var itemId = this.SelectedDataListControl.get("selectedItemId");
      if (!itemId || itemId.length == 0) {
        return;
      }

      var test = this.dataSources.filter(function (item) {
        return item.get("itemId") == itemId;
      });
      if (test.length == 1) {
        this.dataSources.remove(test[0]);
      }
    },

    addAccounts: function (valueList) {
      if (!valueList) {
        return;
      }

      var values = valueList.split("|");
      if (!values || values.length == 0) {
        return;
      }

      var self = this;
      for (var n = 0; n < values.length; n++) {
        var value = values[n].split("^");
        if (!value || value.length == 0)
          continue;

        switch (value[1].toLowerCase()) {
          case "user":
            var userExist = this.users.filter(function (item) {
              return item.get("itemId") == value[0];
            });
            if (userExist.length == 0) {
              $.get(applib.settings.navigation.commandRoutePrefix + "design/getuser?userName=" + value[0]).done(function (data) {
                self.users.add(data);
              });
            }
            break;
          case "role":
            var roleExist = this.roles.filter(function (item) {
              return item.get("itemId") == value[0];
            });
            if (roleExist.length == 0) {
              var role = { itemId: value[0] };
              self.roles.add(role);
            }
            break;
        }
      }
    },

    removeUsers: function () {
      var itemId = this.UsersListControl.get("selectedItemId");
      if (!itemId)
        return;

      var test = this.users.filter(function (item) {
        return item.get("itemId") == itemId;
      });
      if (test.length == 1) {
        this.users.remove(test[0]);
      }
    },

    removeRoles: function () {
      var itemId = this.RolesListControl.get("selectedItemId");
      if (!itemId)
        return;

      var test = this.roles.filter(function (item) {
        return item.get("itemId") == itemId;
      });
      if (test.length == 1) {
        this.roles.remove(test[0]);
      }
    },

    back: function () {
      var ref = applib.getUrlParameter("ref");
      if (ref == "") {
        ref = "DesignManager";
      }

      window.location = applib.settings.navigation.url + ref;
    },

    setDate: function (dateFieldValue, datePickerControl, timePickerControl) {
      if (_sc.Helpers.date.isISO(dateFieldValue)) {
        datePickerControl.set("date", dateFieldValue);
        if (dateFieldValue.indexOf("T") == 8) {
          timePickerControl.set("time", dateFieldValue.substr(8, 7));
        }
      }
    },

    getDate: function (datePickerControl) {
      var date = datePickerControl.get("date");
      if (_sc.Helpers.date.isISO(date)) {
        return _sc.Helpers.date.parseISO(date);
      }

      return null;
    },

    getUtcDateValue: function (dateValue, completed) {
      var isoDate = _sc.Helpers.date.toISO(dateValue);
      if (!isoDate || isoDate.length == 0) {
        completed("");
        return;
      }

      $.ajax({
        url: applib.settings.navigation.commandRoutePrefix + "collection/getutcdate",
        dataType: "json",
        async: false,
        data: { isoDate: isoDate }
      }).done(function (data) {
        completed(data);
      });
    }
  });

  return editCollection;
});