﻿define(["sitecore", "/-/speak/v1/odg/applib.js"], function (_sc, applib) {
  return _sc.Definitions.App.extend({
    initialized: function () {
      $.ajaxSetup({ cache: false });
      var app = this;

      _sc.on("item:updated", this.itemUpdated, this);
      this.on("item:save", this.saveTask, this);
      this.on("change:localProject", this.updateLocalProject, this);

      this.NameTextBox.on("change:text", this.changeName, this);

      this.set("lastSelectedDesignId", null);
      this.on("change:lastSelectedDesignId", function () {
        this.InsertDesignButton.set("isEnabled", this.get("lastSelectedDesignId") != null);
      }, this);

      _sc.on("sc-frame-message", function (value) {
        var modal = this.SelectDesignDialogWindow.viewModel.$el.data('modal');
        if (modal && modal.isShown) {
          this.set("lastSelectedDesignId", value && value.length > 0 ? value : null);
        }
      }, this);

      this.SelectDesignDialogWindow.viewModel.$el.addClass("frameDialog");

      this.ProcessingJobsDataSource.on("change:items", function () {
        var items = this.ProcessingJobsDataSource.get("items");
        this.ProcessingJobsComboBox.set("items", items);
        if (this.ProcessingJobsComboBox.get("selectedItemId") == null && items.length > 0) {
          this.ProcessingJobsComboBox.set("selectedItemId", items[0].itemId);
        }
      }, this);

      var guid = applib.getUrlParameter("id");
      if (guid != "") {
        applib.getItemFromDatabase(guid, loadItem);
        applib.getItemAccess(guid, function (data) {
          this.SaveButton.set("isEnabled", data.canWrite);
          this.SelectDesignButton.set("isEnabled", data.canWrite);
        }, this);
      }
      else {
        this.back();
      }

      this.TabControl.on("change:selectedTab", this.selectedTabChanged, this);
      this.selectEditTab();

      function loadItem(item) {
        if (item == null) {
          app.back();
        }

        app.item = item;
        app.itemModel = item.toModel();
        app.updateInfo();
      }
    },

    changeName: function () {
      this.NameValue.set("text", this.NameTextBox.get("text"));
      this.NameValue.viewModel.$el.attr("title", this.NameTextBox.get("text"));
    },

    selectEditTab: function () {
      var isNew = applib.getUrlParameter("new");
      if (isNew == "1") {
        this.TabControl.viewModel.$el.find(".active-first-sibling").removeClass("active-first-sibling");
        this.TabControl.viewModel.$el.find("> ul li[data-tab-id='{9D847106-FB57-432B-A73C-72E45E1A6EE8}']").nextUntil('li.sc-tabcontrol-header a').first().addClass("active-first-sibling");
        this.TabControl.set("selectedTab", "{9D847106-FB57-432B-A73C-72E45E1A6EE8}");
      }
    },

    selectedTabChanged: function () {
      var isEditTab = this.TabControl.get("selectedTab") == "{9D847106-FB57-432B-A73C-72E45E1A6EE8}";
      var appContentMain = $("section.sc-applicationContent-main");
      var appContentInfo = $("aside.sc-applicationContent-info");

      if (isEditTab == true) {
        appContentInfo.css("display", "none");
        appContentMain.addClass("col-md-12");
        appContentMain.removeClass("col-md-9");
      }
      else {
        appContentInfo.css("display", "block");
        appContentMain.addClass("col-md-9");
        appContentMain.removeClass("col-md-12");
      }
    },

    updateLocalProject: function () {
      var itemId = this.get("localProject") != null ? this.get("localProject") : "";

      if (this.item["Local Project"] !== itemId) {
        this.item["Local Project"] = itemId;

        if (this.designEditor) {
          this.designEditor.loadEditor(this.item);
        }
      }
    },

    updateDesignInfo: function () {
      if (this.selectedDesign != null) {
        this.SelectedDesignImage.set("imageUrl", this.selectedDesign.$mediaurl);
        this.SelectedDesignName.set("text", this.selectedDesign.$displayName);
        this.SelectedDesignName.viewModel.$el.attr("title", this.selectedDesign.$displayName);
        $("[data-sc-id=SelectedDesignBorder]").css("border-color", "");
      }
      else {
        this.SelectedDesignImage.set("imageUrl", "");
        this.SelectedDesignName.set("text", this.NoDesignSelected.get("text"));
        this.SelectedDesignName.viewModel.$el.attr("title", this.NoDesignSelected.get("text"));
      }

      this.SelectedDesignBorder.viewModel.$el.addClass("selected-itemIcon");
    },

    updateInfo: function () {
      if (!this.item || this.item.$templateName != "Design Document") {
        return;
      }

      var self = this;

      var collectionId = this.itemModel.get("Design Collection");
      if (collectionId != "") {

        this.DesignFrame.set("sourceUrl", applib.settings.navigation.url + "components/dialogs/selectdesigndialog?id=" + collectionId);

        applib.getItemFromDatabase(collectionId, function (collectionItem) {
          self.collectionItem = collectionItem;
          self.updateCollectionInfo();
        });
      }

      var fieldValue = this.itemModel.get("__Created by");
      if (fieldValue != "") {
        $.get(applib.settings.navigation.commandRoutePrefix + "design/getuser?userName=" + fieldValue).done(function (data) {
          self.CreatedByValue.set("text", data.fullName != "" ? data.fullName : data.itemId);
          self.EmailValue.set("text", data.email);
        });
      }

      fieldValue = applib.getDateValue(this.item, "__Created");
      this.DateCreatedValue.set("text", fieldValue);

      this.NameValue.set("text", this.item.$displayName);
      this.NameValue.viewModel.$el.attr("title", this.item.$displayName);

      var nameColumn = $("[data-sc-id=ValuesColumn4]");
      if (nameColumn) {
        nameColumn.addClass("iteminfo-name");
      }

      this.NameTextBox.set("text", this.item.$displayName);
      this.InfoTextBox.set("text", this.item.Description);

      this.getLocalProjectStatus();

      applib.getItemFromDatabase(this.item["Design Template"], function (data) {
        self.selectedDesign = data;
        self.updateDesignInfo();
      }, this);

      this.insertRendering(applib.settings.renderings.editDesignDialog, { path: applib.settings.renderings.renderingRootPath, $el: this.DesignEditorBorder.viewModel.$el }, function (designEditor) {
        self["designEditor"] = designEditor;
        designEditor.loadEditor(self.item);
      });
    },

    updateCollectionInfo: function () {
      if (this.collectionItem != null) {
        this.CollectionHyperlinkButton.set("text", this.collectionItem.$displayName);
        this.CollectionHyperlinkButton.viewModel.$el.attr("title", this.collectionItem.$displayName);

        var collectionColumn = $("[data-sc-id=ValuesColumn8]");
        if (collectionColumn) {
          collectionColumn.addClass("iteminfo-name");
        }
      }
      else {
        this.SelectDesignButton.set("isEnabled", false);
        this.LinkBorder.set("isVisible", false);
        this.MessageBorder.set("isVisible", true);
      }
    },

    saveTask: function () {
      this.SaveDocumentMessageBar.removeMessages();

      var name = this.NameTextBox.get("text");

      if (name && this.selectedDesign != null) {
        this.isSaving = true;
        this.ProgressIndicator.set("isBusy", true);

        this.itemModel.set("__Display name", name);
        this.itemModel.set("Name", name);
        this.itemModel.set("Description", this.InfoTextBox.get("text"));

        this.cloneProject = this.itemModel.get("Design Template") != this.selectedDesign.itemId;
        this.updateUsageIds = [];
        if (this.cloneProject) {
          this.updateUsageIds = [this.itemModel.get("Design Template"), this.selectedDesign.itemId];
        }
        this.itemModel.set("Design Template", this.selectedDesign.itemId);
        this.itemModel.save();
      } else {
        if (!name) {
          $("[data-sc-id=NameTextBox]").css("border-color", "red");
        }

        if (!this.selectedDesign) {
          this.MessageBar.set("isVisible", true);
          $("[data-sc-id=SelectedDesignBorder]").css("border-color", "red");
        }
      }
    },

    openSelectDesign: function () {
      var showDialog = true;
      if (this.item["Design Template"] != "") {
        var changeConfirmationText = this.ChangeDesignConfirmationText.get("text");
        if (!confirm(changeConfirmationText)) {
          showDialog = false;
        }
      }
      if (showDialog) {
        this.set("lastSelectedDesignId", null);
        this.SelectDesignDialogWindow.show();
      }
    },

    selectDesignSelected: function () {
      this.SelectDesignDialogWindow.hide();
      var app = this;

      if (this.get("lastSelectedDesignId") != null) {
        applib.getItemFromDatabase(this.get("lastSelectedDesignId"), function (designItem) {
          app.selectedDesign = designItem;
          app.updateDesignInfo();
        });
      }
    },

    getLocalProjectStatus: function () {
      var self = this;
      var ajaxOptions = {
        url: applib.settings.navigation.commandRoutePrefix + "design/gettasklocalprojectstatus",
        type: "GET",
        dataType: "json",
        data: { itemId: this.item.itemId }
      };
      $.ajax(ajaxOptions).done(function (data) {
        if (data.localProject) {
          self.set("localProject", data.localProject);
        }
        if (data.needCloning === true) {
          self.cloneTaskProject();
        }
        if (data.needSync) {
          self.SyncProjectButton.set("isEnabled", true);
        }
        if (data.designTemplateChanged) {
          self.ChangeProjectButton.set("isEnabled", true);
        }
      }).fail(function (err) {
      });
    },

    syncTaskLocalProject: function (clone, sync) {
      var self = this;
      var ajaxOptions = {
        url: applib.settings.navigation.commandRoutePrefix + "design/synctasklocalproject",
        type: "GET",
        dataType: "json",
        data: {
          itemId: this.item.itemId,
          clone: clone,
          sync: sync
        }
      };

      var cloneOnSave = this.cloneProject;
      this.ProgressIndicator.set("isBusy", sync || (clone && this.cloneProject));

      $.ajax(ajaxOptions).done(function (data) {
        self.ProgressIndicator.set("isBusy", false);
        if (cloneOnSave) {
          self.SaveDocumentMessageBar.addMessage("notification", self.ChangesSavedText.get("text"));
        }
        self.set("localProject", data);
        if (clone) {
          self.ChangeProjectButton.set("isEnabled", data == null);
        }
        if (sync) {
          self.SyncProjectButton.set("isEnabled", data == null);
          if (self.designEditor) {
            self.designEditor.loadEditor(self.item);
          }
        }
      }).fail(function (err) {
        self.ProgressIndicator.set("isBusy", false);
        self.SaveDocumentMessageBar.addMessage("error", err.responseText);

      });
    },

    syncTaskProject: function () {
      this.syncTaskLocalProject(false, true);
    },

    cloneTaskProject: function () {
      this.syncTaskLocalProject(true, false);
    },

    itemUpdated: function (value) {
      if (this.item != null && applib.isUpdatedItem(this.item.itemId, value.url)) {
        this.isSaving = false;
        this.ProgressIndicator.set("isBusy", false);
        if (this.cloneProject) {
          this.cloneTaskProject();
          this.cloneProject = false;
        }
        else if (value.data && value.data.length > 0) {
          this.SaveDocumentMessageBar.addMessage("notification", this.ChangesSavedText.get("text"));
        }

        if (this.updateUsageIds && this.updateUsageIds.length > 0) {
          _.each(this.updateUsageIds, function (itemId) {
            $.ajax({
              url: applib.settings.navigation.commandRoutePrefix + "design/updateusage",
              type: "PUT",
              dataType: "json",
              data: { itemId: itemId }
            });
          });
        }
      }
    },

    sendToPrint: function () {
      this.DesignSentMessageBar.set("isVisible", false);
      this.ErrorMessageBar.removeMessages();
      var processingJobId = this.ProcessingJobsComboBox.get("selectedItemId");
      var self = this;
      $.ajax({
        url: applib.settings.navigation.commandRoutePrefix + "design/publishpdf",
        type: "POST",
        dataType: "json",
        data: {
          itemId: this.item["Local Project"],
          processingJobId: processingJobId,
          lang: this.PublishLanguageSelector.get("contentLanguage")
        }
      }).done(function () {
        self.DesignSentMessageBar.set("isVisible", true);
      }).fail(function (err) {
        self.ErrorMessageBar.addMessage("error", err.statusText);
      });
    },

    viewCollection: function () {
      var ref = applib.getUrlParameter("ref");
      if (ref == "") {
        ref = "DocumentPublisher";
      }

      if (this.collectionItem != null) {
        window.location = applib.settings.navigation.url + "salesviewcollection?id=" + this.collectionItem.itemId + "&ref=" + ref;
      }
    },

    back: function () {
      var ref = applib.getUrlParameter("ref");
      if (ref == "") {
        ref = "DocumentPublisher";
      }

      window.location = applib.settings.navigation.url + ref;
    }
  });
});