﻿define(["sitecore", "/-/speak/v1/odg/applib.js"], function (_sc, applib) {
  var viewCollection = _sc.Definitions.App.extend({
    initialized: function () {
      $.ajaxSetup({ cache: false });
      var app = this;
      this.on("closePopup", this.closePopup, this);
      applib.addDialog(this, applib.settings.renderings.createCollectionDialog, "createDialog");

      this.itemGuid = applib.getUrlParameter("id");
      applib.getItemFromDatabase(this.itemGuid, loadItem);

      this.ResolutionComboBox.on("change:selectedItemId", this.loadChart, this);
      this.getChartResolution();

      this.NoTopDesignsBorder.viewModel.$el.addClass("top-design-message"); // center vertical
      function loadItem(item) {
        if (item == null) {
          app.back();
        }

        app.item = item;
        app.itemModel = item.toModel();
        app.updateInfo();
      }

      applib.getItemAccess(applib.settings.rootFolderIds.designCollections, function (data) {
        this.CreateCollectionButton.set("isEnabled", data.canCreate);
      }, this);

      this.DesignsListControl.on("change:selectedItemId", this.setInfoPanel, this);
    },

    getChartResolution: function () {

      var self = this;
      $.ajax({
        url: applib.settings.navigation.commandRoutePrefix + "collection/getchartresolution",
        dataType: "json"
      }).done(function (data) {
        self.ResolutionComboBox.set("items", data.data);
      });
    },

    loadChart: function () {
      this.ChartDataProvider.set("resolution", this.ResolutionComboBox.get("selectedValue"));

      var serverRequestOptions = {};
      serverRequestOptions["url"] = applib.settings.navigation.commandRoutePrefix + "collection/getchart";

      serverRequestOptions.parameters = {
        "itemId": this.itemGuid
      };
      this.ChartDataProvider.viewModel.getData(serverRequestOptions);
      this.ChartDataProvider.on("error", function (errorObject) {
        this.ChartDataProvider.set("isBusy", false);
      }, this);
    },

    updateInfo: function () {
      if (!this.item || this.item.$templateName != "Design Collection") {
        return;
      }
      
      this.ItemInfoContent.set("item", this.item);
    },

    setInfoPanel: function () {
      var selected = this.DesignsListControl.get("selectedItemId");
      if (selected && selected.length > 0) {
        applib.getItemFromList(this.DesignsListControl, this.fillDesignInfo, this);
      }
    },

    fillDesignInfo: function (item) {
      this.DesignInfoContent.set("item", item);
    },

    toggleView: function () {
      var isDetailsList = this.DetailListToggleButton.get("isEnabled") && this.DetailListToggleButton.get("isOpen");
      if (isDetailsList) {
        this.DesignsListControl.viewModel.setDetailListView();
      } else {
        this.DesignsListControl.viewModel.setIconView();
      }

      this.DetailListToggleButton.set("isEnabled", !isDetailsList);
      this.DetailListToggleButton.set("isOpen", isDetailsList);
      this.IconListToggleButton.set("isEnabled", isDetailsList);
      this.IconListToggleButton.set("isOpen", !isDetailsList);
    },

    editCollection: function () {
      var ref = applib.getUrlParameter("ref");
      if (ref == "") {
        ref = "DesignManager";
      }

      window.location = applib.settings.navigation.url + "editcollection?id=" + this.item.itemId + "&ref=" + ref;
    },

    back: function () {
      var ref = applib.getUrlParameter("ref");
      if (ref == "") {
        ref = "DesignManager";
      }

      window.location = applib.settings.navigation.url + ref;
    }
  });

  return viewCollection;
});